"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseBytes = void 0;
// IEC Sizes.
// kibis of bits
var IECByteSizes;
(function (IECByteSizes) {
    IECByteSizes[IECByteSizes["Byte"] = 1] = "Byte";
    IECByteSizes[IECByteSizes["KiByte"] = 1024] = "KiByte";
    IECByteSizes[IECByteSizes["MiByte"] = 1048576] = "MiByte";
    IECByteSizes[IECByteSizes["GiByte"] = 1073741824] = "GiByte";
    IECByteSizes[IECByteSizes["TiByte"] = 1099511627776] = "TiByte";
    IECByteSizes[IECByteSizes["PiByte"] = 1125899906842624] = "PiByte";
    IECByteSizes[IECByteSizes["EiByte"] = 1152921504606847000] = "EiByte";
})(IECByteSizes || (IECByteSizes = {}));
// SI Sizes.
var SIByteSizes;
(function (SIByteSizes) {
    SIByteSizes[SIByteSizes["IByte"] = 1] = "IByte";
    SIByteSizes[SIByteSizes["KByte"] = 1000] = "KByte";
    SIByteSizes[SIByteSizes["MByte"] = 1000000] = "MByte";
    SIByteSizes[SIByteSizes["GByte"] = 1000000000] = "GByte";
    SIByteSizes[SIByteSizes["TByte"] = 1000000000000] = "TByte";
    SIByteSizes[SIByteSizes["PByte"] = 1000000000000000] = "PByte";
    SIByteSizes[SIByteSizes["EByte"] = 1000000000000000000] = "EByte";
})(SIByteSizes || (SIByteSizes = {}));
const bytesSizeMap = new Map([
    ['kib', IECByteSizes.KiByte],
    ['kb', SIByteSizes.KByte],
    ['mib', IECByteSizes.MiByte],
    ['mb', SIByteSizes.MByte],
    ['gib', IECByteSizes.GiByte],
    ['gb', SIByteSizes.GByte],
    ['tib', IECByteSizes.TiByte],
    ['tb', SIByteSizes.TByte],
    ['pib', IECByteSizes.PiByte],
    ['pb', SIByteSizes.PByte],
    ['eib', IECByteSizes.EiByte],
    ['eb', SIByteSizes.EByte],
    // without suffixes
    ['ki', IECByteSizes.KiByte],
    ['k', SIByteSizes.KByte],
    ['mi', IECByteSizes.MiByte],
    ['m', SIByteSizes.MByte],
    ['gi', IECByteSizes.GiByte],
    ['g', SIByteSizes.GByte],
    ['ti', IECByteSizes.TiByte],
    ['t', SIByteSizes.TByte],
    ['pi', IECByteSizes.PiByte],
    ['p', SIByteSizes.PByte],
    ['ei', IECByteSizes.EiByte],
    ['e', SIByteSizes.EByte],
    // keep these last because they will match previous suffixes
    ['b', IECByteSizes.Byte],
    ['', IECByteSizes.Byte],
]);
// parseBytes parses a string representation of bytes into the number
// of bytes it represents.
//
// parseBytes("42 MB") -> 42000000
// parseBytes("42 mib") -> 44040192
function parseBytes(input) {
    // remove thousand (,) notation
    const normalizedInput = input.replace(/,/g, '').trim().toLowerCase();
    const suffixes = Array.from(bytesSizeMap.keys());
    const unit = suffixes.find(suffix => {
        return normalizedInput.endsWith(suffix);
    });
    if (unit === undefined) {
        throw new Error(`No such unit: ${unit}`);
    }
    const numPart = normalizedInput.replace(unit, '').trim();
    const num = parseFloat(numPart);
    const multiplier = bytesSizeMap.get(unit);
    if (!multiplier) {
        throw new Error(`No multiplier for unit: ${unit}`);
    }
    return num * multiplier;
}
exports.parseBytes = parseBytes;
